
#include <boost/marshal/server.hpp>
#include <boost/marshal/client.hpp>
#include <boost/marshal/marshal.hpp>
#include <boost/marshal/call.hpp>
#include <boost/marshal/simple_acceptor.hpp>
#include <boost/marshal/simple_connector.hpp>

void nothing()
{
}

int inc(int i)
{
	return ++i;
}

int add2( int i, int j )
{
	return i+j;
}

namespace marshal = boost::marshal;
using namespace boost::asio;

void network_marshal_test()
{
	marshal::registry<int> reg;

	reg.set<void ()>(0, nothing); // register function
	reg.set<int (int)>(1, inc); // register function
	reg.set<int (int, int)>(2, add2); // register function

	marshal::server<marshal::registry<int>, marshal::simple_acceptor> server(reg, 1097);

	marshal::client<marshal::registry<int>, marshal::simple_connector> client(
		ip::tcp::endpoint(ip::address::from_string("127.0.0.1"), 1097));

	// embed the function id into the call as well as two parameters
	marshal::call<int (int, int)> call_5_6(5,6);
	marshal::call<int (int)> call_1(1);
	marshal::call<void ()> call_;

	client(0, call_, marshal::call_options(
		marshal::call_options::completed_only,
		marshal::call_options::sync));

	BOOST_CHECK(call_.has_completed());

	client(1, call_1, marshal::call_options(
		marshal::call_options::return_only,
		marshal::call_options::sync));

	BOOST_CHECK(call_1.has_completed());
	BOOST_CHECK(call_1.return_val() == 2);

	client(2, call_5_6, marshal::call_options(
		marshal::call_options::return_only,
		marshal::call_options::async));

	while(!call_5_6.has_completed())
	{}
	BOOST_CHECK(call_5_6.return_val() == 11);
	
}
