#ifndef BOOST_MARSHAL_CALL_HPP
#define BOOST_MARSHAL_CALL_HPP

#include <boost/marshal/config.hpp>
#include <boost/detail/arity.hpp>

namespace boost {
namespace marshal {

///	Base class for marshal::call
class call_base
{
public:
	volatile bool has_completed() const {return completed;}
	void complete() {completed = true;}
	///	Returns the serialized call parameters
	virtual const std::string &parameters() const = 0;
	virtual void result_string(const std::string &str, const call_options &options) = 0;
protected:
	call_base()
	{
		reset();
	}
	void reset()
	{
		completed = false;
	}

private:
	volatile bool completed;
};

template<typename Signature, typename ArchivePair = binary_archive,
	typename Enable=void, typename Enable2=void>
class call;

#define BOOST_ARITY_SEPARATE_VOID_RETURN
#define BOOST_ARITY_ITERATION_PARAMS \
	(3,(0,BOOST_MARSHAL_MAX_ARGS,<boost/marshal/detail/call_template.hpp>))
#include <boost/detail/arity_iterate.hpp>
#undef BOOST_ARITY_ITERATION_PARAMS
#undef BOOST_ARITY_SEPARATE_VOID_RETURN

} // namespace marshal
} // namespace boost

#endif // BOOST_MARSHAL_CALL_HPP