#ifndef BOOST_MARSHAL_REGISTRY
#define BOOST_MARSHAL_REGISTRY

#include <boost/ptr_container/ptr_map.hpp>
#include <boost/call_traits.hpp>
#include <boost/marshal/function.hpp>
#include <boost/marshal/marshaled.hpp>

namespace boost {
namespace marshal {

template<typename Id, typename ArchivePair = binary_archive>
class registry
{
public:
	typedef Id id_type;
	typedef ArchivePair archive_type;

	template<typename Signature>
	// I'd like to use call traits param type for id but ptr_map complains on Id=int
	void set(Id id, const boost::function<Signature> &f)
	{
		reg.insert(id, new marshaled<Signature>(f));
	}
	void operator()(typename boost::call_traits<Id>::param_type id,
		const std::string &params, const call_options &options, std::string *result = NULL)
	{
		std::stringstream p_stream(params, std::ios::in | std::ios::out | std::ios::binary);
		typename archive_type::iarchive_type p_archive(p_stream);

		if (options.marshal_option >= call_options::return_only)
		{
			std::stringstream r_stream(std::ios::in | std::ios::out | std::ios::binary);
			typename archive_type::oarchive_type r_archive(r_stream);
			reg[id].call(options, p_archive, &r_archive);
			*result = r_stream.str();
		}
		else
			reg[id].call(options, p_archive, NULL);
	}
private:
	boost::ptr_map<Id, detail::function<ArchivePair> > reg;
};

} // namespace marshal
} // namespace boost

#endif // BOOST_MARSHAL_REGISTRY
