#ifndef BOOST_MARSHAL_SERVER_HPP
#define BOOST_MARSHAL_SERVER_HPP

#include <boost/marshal/detail/threaded_service.hpp>
#include <boost/ptr_container/ptr_vector.hpp>
#include <boost/marshal/registry_server.hpp>

namespace boost {
namespace marshal {

template<typename Registry, typename Acceptor>
class server : public detail::threaded_service
{
public:
	server(Registry &reg, int port)
		: registry_(reg)
		, acceptor(service, port, boost::bind(&server<Registry, Acceptor>::connected, this, _1))
	{
		start_service_thread();
	}
private:
	void connected(std::auto_ptr<boost::asio::ip::tcp::socket> socket_ptr)
	{
		servers.push_back(new registry_server<Registry>(registry_,
			boost::shared_ptr<boost::asio::ip::tcp::socket>(socket_ptr)));
	}

	Acceptor acceptor;

	Registry &registry_;
	boost::ptr_vector<registry_server<Registry> > servers;
};

} // namespace marshal
} // namespace boost

#endif // BOOST_MARSHAL_SERVER_HPP